# Neo react changelog

This file details all new features and changes to functionality. If you only need to see changes that require updates to your project, see the [breaking changes](./Breaking.md) readme.

## Version 1.3.0

- ViewBase and ViewComponentBase now require that the linked view model has a `TaskRunner` as the first argument. See breaking changes for more info.
- GridLayout
    - Added `arrangeVertically` prop. Allows items to flow from top to bottom instead of left to right. Useful the height of each item differs.
- Neo.Card
    - Now has a `footerContent` prop. This allows a footer to be added to the card. The footer will be anchored to the bottom of the card, and will be collapsed if the card is collapsed.
- Settings
    - Added `buttons.backButton` setting to replace the `icons.backIcon` global setting. This allows more customisation of back buttons used in built in neo screens.
    - Added `icons.unDelete` to allow changing the undo delete icon in neo grids.
- Deferred deleting: see below.

### Deferred deleting

Previously when clicking the default delete button in a neo grid, the item would be removed from the bound list, and tracked under the `deletedList` property. When serialising the list (usually on clicking save), the deleted items would be included with a tracking state of "deleted" so that they could be deleted from the database.

This was slightly confusing for a user, since they might not know that a save is required to actually commit the delete. Additionally, if the save failed, the item would not re-appear in the grid and the user would be stuck, having to reload the screen.

In this version, a new option has been added to allow the grid to keep displaying the row being deleted, but displayed with ~~strikethrough~~ text and 50% transparency. The delete button will show a different icon when the item is marked as deleted, allowing the user to undo the delete before saving. A tooltip will also show when hovering over the row.

To make this work, 3 main changes were made to `ModelBase` and `List`. 
- `ModelBase` now has an `isSelfDeleting` property. 
    - This causes the grid to display the item differently. 
    - Models with this property set to true will be serialised with a tracking state of deleted.
    - This will cause `isDirty` to return true, however `isSelfDirty` will remain false.
- `List` has new methods: `beginRemove` and `toggleRemove`.
    - These new methods check whether the list is in "soft delete" mode (see below), and if so, sets `isSelfDeleting` to true on the item being removed.
    - The existing `remove` and `removeWithoutTracking` methods will always remove the item from the main list regardless of the delete mode.
- `List` now has an `activeLength` property.
    - Use this if you need to check if the list is required to have a certain amount of items. E.g. in validation rules.


#### Settings:

To control the behavior when removing an item, you use the global setting, or you can configure each list individually by passing in options to the list constructor. 

- To configure globally, set `Misc.Settings.list.removeItemBehavior` to `RemoveItemBehavior.MarkDeleted`.
- The tooltip to show that the item is being deleted can be customised by setting the `isDeletingTooltip` prop on the grid row, or by using the `Misc.Settings.grid.isDeletingTooltip` global setting.

By default this setting will be enabled for catalogue grids, but is not enabled globally.

To change the styling of deleted rows, target: `tr.neo-item-deleting`.