import { Misc, Utils } from "@singularsystems/neo-core";
import React from "react";
var BootstrapUtils = /** @class */ (function () {
    function BootstrapUtils() {
        this.breakPoints = null;
    }
    Object.defineProperty(BootstrapUtils.prototype, "isPre5", {
        /** Bootstrap version is before v5 */
        get: function () {
            return Misc.Settings.bootstrap.version < 5;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BootstrapUtils.prototype, "marginS", {
        /** Class prefix (without -) for start margin / left margin. */
        get: function () {
            return this.isPre5 ? "ml" : "ms";
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BootstrapUtils.prototype, "marginE", {
        /** Class prefix (without -) for end margin / right margin. */
        get: function () {
            return this.isPre5 ? "mr" : "me";
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Gets the close button used in component headers.
     * @param pre5ClassName Class to add if bootstrap version is <5.
     */
    BootstrapUtils.prototype.getCloseButton = function (onClick, pre5ClassName) {
        pre5ClassName = Utils.joinWithSpaces("close", pre5ClassName);
        return this.isPre5 ?
            React.createElement("button", { type: "button", className: pre5ClassName, "aria-label": "Close", onClick: onClick },
                React.createElement("span", { "aria-hidden": "true" }, "\u00D7")) :
            React.createElement("button", { type: "button", className: "btn-close", "aria-label": "Close", onClick: onClick });
    };
    BootstrapUtils.prototype.getBreakpoints = function () {
        if (!this.breakPoints) {
            var styles = window.getComputedStyle(document.body);
            this.breakPoints = {
                xs: parseInt(styles.getPropertyValue('--bs-breakpoint-xs')),
                sm: parseInt(styles.getPropertyValue('--bs-breakpoint-sm')),
                md: parseInt(styles.getPropertyValue('--bs-breakpoint-md')),
                lg: parseInt(styles.getPropertyValue('--bs-breakpoint-lg')),
                xl: parseInt(styles.getPropertyValue('--bs-breakpoint-xl')),
                xxl: parseInt(styles.getPropertyValue('--bs-breakpoint-xxl'))
            };
        }
        return this.breakPoints;
    };
    BootstrapUtils.prototype.currentDimension = function (props) {
        var _a, _b, _c, _d, _e;
        var width = window.innerWidth;
        var breakpoints = this.getBreakpoints();
        var found = props.xs;
        if (width < breakpoints.sm)
            return found;
        found = (_a = props.sm) !== null && _a !== void 0 ? _a : found;
        if (width < breakpoints.md)
            return found;
        found = (_b = props.md) !== null && _b !== void 0 ? _b : found;
        if (width < breakpoints.lg)
            return found;
        found = (_c = props.lg) !== null && _c !== void 0 ? _c : found;
        if (width < breakpoints.xl)
            return found;
        found = (_d = props.xl) !== null && _d !== void 0 ? _d : found;
        if (width < breakpoints.xxl)
            return found;
        return (_e = props.xxl) !== null && _e !== void 0 ? _e : found;
    };
    return BootstrapUtils;
}());
export default new BootstrapUtils();
