import { __decorate, __extends, __metadata } from "tslib";
import React from 'react';
import { observer } from 'mobx-react';
import { NeoReactTypes } from '../Modules/Types';
import { Misc } from '@singularsystems/neo-core';
var ColorPicker = /** @class */ (function (_super) {
    __extends(ColorPicker, _super);
    function ColorPicker(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {
            visible: false,
            selectedColor: _this.props.bind.value
        };
        _this.handleChange = function (color) {
            // Set the color value
            _this.setState({ selectedColor: color });
        };
        _this.visibleToggle = function () {
            _this.setState({ visible: !_this.state.visible });
        };
        _this.cancel = function () {
            // Reset the color
            _this.setState({ selectedColor: _this.props.bind.value });
            _this.visibleToggle();
        };
        _this.accept = function () {
            _this.props.bind.value = _this.state.selectedColor;
            _this.visibleToggle();
            // Check if there is a custom event
            if (_this.props.onChangeEvent) {
                _this.props.onChangeEvent();
            }
        };
        _this.handleChange = _this.handleChange.bind(_this);
        return _this;
    }
    ColorPicker_1 = ColorPicker;
    ColorPicker.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.bind.value !== this.state.selectedColor) {
            // Update the state selectedColor if the color that was in the bind was changed
            this.setState({ selectedColor: nextProps.bind.value });
        }
    };
    ColorPicker.prototype.componentDidMount = function () {
        this.setState({ selectedColor: this.props.bind.value });
    };
    ColorPicker.prototype.render = function () {
        var _this = this;
        var style = { backgroundColor: this.props.bind.value };
        return (React.createElement("div", { className: "color-picker" },
            React.createElement("div", { className: "mode-overlay" },
                React.createElement("div", { className: 'block-button', style: style, onClick: function () { return _this.visibleToggle(); } }),
                this.state.visible &&
                    React.createElement(React.Fragment, null,
                        React.createElement("div", { className: "overlay-area", onClick: function () { return _this.cancel(); } }),
                        React.createElement("div", { className: "picker-container" },
                            React.createElement("div", { className: "picker-block" },
                                ColorPicker_1.render(this.state.selectedColor, this.handleChange),
                                React.createElement("div", { className: "action-container" },
                                    React.createElement("div", { className: "btn btn-primary btn-sm", onClick: function () { return _this.accept(); } }, "Accept"),
                                    React.createElement("div", { className: "btn btn-secondary btn-sm ml-1", onClick: function () { return _this.cancel(); } }, "Cancel"))))))));
    };
    /** Renders the inner color picker components without the modal */
    ColorPicker.render = function (color, onChange) {
        var sketchPicker = Misc.Globals.appService.tryGet(NeoReactTypes.Components.SketchPicker);
        var colorPicker = Misc.Globals.appService.tryGet(NeoReactTypes.Components.ColorPicker);
        if (sketchPicker && colorPicker) {
            throw Error("SketchPicker and ColorPicker are registered. Make sure only 1 is installed, and registered.");
        }
        if (sketchPicker) {
            return React.createElement(sketchPicker, { color: color, onChange: function (c) { return onChange(c.hex); } });
        }
        if (colorPicker) {
            return React.createElement(colorPicker, { color: color, onChange: onChange });
        }
        return React.createElement("input", { type: "color", value: color, onInput: function (e) { return onChange(e.target.value); } });
    };
    var ColorPicker_1;
    ColorPicker = ColorPicker_1 = __decorate([
        observer,
        __metadata("design:paramtypes", [Object])
    ], ColorPicker);
    return ColorPicker;
}(React.Component));
export default ColorPicker;
