import React, { JSX } from 'react';
import { Model } from '@singularsystems/neo-core';
import { AxiosPromise } from 'axios';
import { DropDownBase, ItemSource } from './DropDownBase';
import type { Props } from 'react-select';
import { ICommonEditorProps } from '../PropTypes';
import { FormContext } from '../FormContext';
interface IOption {
    value: string | number;
    label: string;
    item?: any;
}
interface IAutoCompleteCommonProps<T> extends ICommonEditorProps, Props<IOption, true> {
    /**
     * Callback for loading data.
     */
    itemSource?: (searchValue: string) => (Promise<T[]> | AxiosPromise<T[]>);
    /**
     * Items to display for non async drop down, or default items to display in async drop down before typing.
     */
    items?: ItemSource<T>;
    valueMember?: keyof T;
    displayMember?: keyof T;
    /**
     * Function used to filter items out of the datasource.
     * **Note:** this will only apply to the `items` prop. */
    filterPredicate?: (item: T) => boolean;
    /**
     * Function used to filter items based on what the user has typed.
     * Applies after `filterPredicate`, and is ignored if `itemSource` is specified.
     * Use if you are displaying multiple columns and need to match on any of the columns.
     */
    inputFilter?: (input: string, item: T) => boolean;
    /**
     * Property of an item to use as the description.
     *
     * Description will be shown in muted text below the item name.
     */
    descriptionMember?: keyof T | false;
    /**
     * Callback used to provide a custom option component instead of displaying the display member value.
     */
    option?: (item: T) => JSX.Element;
    /**
     * Text to display when the user has typed something, but no records were returned.
     */
    noRecordsPrompt?: string;
    /**
     * Text to display when the user clicks in the text area, and no search value has been entered.
     */
    typeToSearchPrompt?: string;
    /**
     * Callback which fires when an item is selected.
     * The removedItem parameter will be provided when removing an item from a multi select drop down.
     * The addedItem parameter will be provided when an item is selected in a multi select drop down. This is the item stored in the bindItems list.
     */
    onItemSelected?: (item?: T, removedItem?: Model.ISelectedItem, addedItem?: Model.ISelectedItem) => void;
    /**
     * Millisecond delay from the last typed character, to when the itemSource function is called.
     * Default is 500ms.
     */
    delayMs?: number;
    /**
     * Minimum no of characters a user must type before a search is performed.
     * Default is 1.
     */
    minCharacters?: number;
}
export interface IAutoCompleteProps {
    bind: Model.IPropertyInstance;
    bindDisplay?: Model.IPropertyInstance;
}
export interface IAutoCompleteMultiSelectProps {
    /**
     * A list of ids and labels to use as the selected list.
     */
    bindItems: Model.IPropertyInstance<Model.ISelectedItem[]>;
    /**
     * Optional list of ids. This will be kept in sync with the bindItems prop.
     */
    bindIds?: (string | number)[] | Model.IPropertyInstance<number[]>;
    /**
     * The max no of items to record in the field info if bound to a field info class. (Defaults to 5)
     * If the selected items exceeds this limit, the field info will be recorded as "x items selected".
     */
    fieldInfoLimit?: number;
}
export default class AutoCompleteDropDown<T> extends DropDownBase<T, IAutoCompleteCommonProps<T> & (IAutoCompleteProps | IAutoCompleteMultiSelectProps)> {
    static contextType: React.Context<import("../FormContext").FormContextParams>;
    context: React.ContextType<typeof FormContext>;
    private asyncSelectImplementation;
    private selectImplementation;
    constructor(props: IAutoCompleteCommonProps<T> & (IAutoCompleteProps | IAutoCompleteMultiSelectProps));
    private bindProperty;
    private bindIdsList?;
    private bindIdsProperty?;
    static getBindProp<T>(props: IAutoCompleteProps | IAutoCompleteMultiSelectProps): Model.IPropertyInstance<any>;
    private onKeyPress;
    private setBindProperty;
    private mapTArrayToOptions;
    private onItemSelected;
    render(): string | number | bigint | boolean | Iterable<React.ReactNode> | Promise<string | number | bigint | boolean | React.ReactPortal | React.ReactElement<unknown, string | React.JSXElementConstructor<any>> | Iterable<React.ReactNode> | null | undefined> | JSX.Element | null | undefined;
    private getOption;
}
export {};
