import * as React from 'react';
import { IBindableEditorProps } from '../PropTypes';
import { Data } from '@singularsystems/neo-core';
import { DropDownBase } from './DropDownBase';
import { FormContext } from '../FormContext';
export interface IDropDownSpecificProps<T> {
    items?: T[] | Promise<T[]>;
    itemSource?: Data.IAsyncDataSource<T[]> | Data.IArrayCacheEntry<T>;
    valueMember?: keyof T;
    displayMember?: keyof T;
    /**
     * Set to true if your bind property is nullable, and you need to show a blank item in the list.
     * If your property value is null, a blank item will automatically be shown.
     */
    allowNulls?: boolean;
    /** Function used to filter items out of the datasource. */
    filterPredicate?: (item: T) => boolean;
    /** Callback which fires when an item is selected. */
    onItemSelected?: (item?: T) => void;
    /**
     * True if the display member of the selected item should display as plain text.
     * Use this if you need a readonly display, and don't want a form control rendered.
     */
    renderAsText?: boolean;
    /** Text to display when the bound value is null, and there is no item with null as it's value member. */
    nullText?: string;
    /** Text to display as the first item in the drop down list when the user is allowed to select a blank item. */
    deSelectText?: string;
    /** Set to true if the property should be sorted by value instead of display text. */
    sortByValue?: boolean;
}
export interface IDropDownProps<T> extends IBindableEditorProps<HTMLSelectElement> {
    select: IDropDownSpecificProps<T>;
}
export default class DropDown<T> extends DropDownBase<T, IDropDownProps<T>> {
    static contextType: React.Context<import("../FormContext").FormContextParams>;
    context: React.ContextType<typeof FormContext>;
    private selectedItem;
    private sortByValue?;
    private listIndexSymbol;
    constructor(props: IDropDownProps<T>);
    private get items();
    private createLookupIndex;
    private getSelectedOption;
    private onItemSelected;
    render(): React.ReactElement<unknown, string | React.JSXElementConstructor<any>> | React.JSX.Element;
}
