import { __assign, __awaiter, __decorate, __extends, __generator, __rest } from "tslib";
import * as React from 'react';
import { observer, Observer } from 'mobx-react';
import { Validation, Model, ModalUtils, Misc } from '@singularsystems/neo-core';
import ValidationSummary from './ValidationSummary';
import { reaction } from 'mobx';
import { FormContextProvider } from './FormContext';
var Form = /** @class */ (function (_super) {
    __extends(Form, _super);
    function Form() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Form.prototype.getModelOrModels = function () {
        var model = this.props.model;
        if (model) {
            // Allow binding to ObservableProperty
            if (model.attachedTo && !(model instanceof Array)) {
                return model.attachedTo;
            }
            else {
                return model;
            }
        }
        return undefined;
    };
    Form.prototype.onSubmit = function (e) {
        return __awaiter(this, void 0, void 0, function () {
            var models, isFirstSubmit, isValid;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        e.preventDefault();
                        e.stopPropagation();
                        models = this.getModelOrModels();
                        isFirstSubmit = true;
                        if (models) {
                            Model.GraphTraverser.traverseItems(models, new Model.GraphTraverser.RecursionState(), function (item, state) {
                                if (item.validator.forceShowErrors) {
                                    isFirstSubmit = false;
                                }
                                else {
                                    item.validator.forceShowErrors = true;
                                }
                            });
                        }
                        return [4 /*yield*/, this.validateAsync()];
                    case 1:
                        isValid = _a.sent();
                        if (isValid) {
                            if (this.props.onSubmit)
                                this.props.onSubmit(e);
                        }
                        else {
                            if (this.props.onInvalid)
                                this.props.onInvalid(e, isFirstSubmit);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /** Checks if the model is valid, waiting for any async rules to complete. Returns true if the model is valid. */
    Form.prototype.validateAsync = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var _a;
            _this.disposeReaction();
            var isValid = true;
            var modelOrModels = _this.getModelOrModels();
            var models = modelOrModels === undefined ? [] : (modelOrModels instanceof Array ? modelOrModels : [modelOrModels]);
            if (models) {
                var allValid_1 = function () { return models.every(function (c) { return c.isValid; }); };
                var someBusy = function () { return models.some(function (c) { return c.validator.isBusy; }); };
                if (!allValid_1()) {
                    isValid = false;
                    var codeModal_1 = null;
                    if (_this.props.showSummaryModal || _this.props.invalidSummaryText || _this.props.invalidSummaryTitle) {
                        var onClose = _this.disposeReaction.bind(_this);
                        var closeButton = (_a = Misc.Settings.form.validationModalCloseButton) !== null && _a !== void 0 ? _a : { text: "Ok" };
                        if (closeButton) {
                            closeButton.onClick = onClose;
                        }
                        codeModal_1 = ModalUtils.showModal(_this.props.invalidSummaryTitle || "Invalid data", React.createElement("div", null,
                            _this.props.invalidSummaryText || "Cannot continue due to the following issues:",
                            React.createElement(Observer, null, function () { return React.createElement(ValidationSummary, { graph: Validation.getValidationSummary(modelOrModels), hideTopLevelTitle: !(modelOrModels instanceof Array) }); })), { onClose: onClose, closeButton: closeButton });
                    }
                    if (someBusy()) {
                        // Async rule is busy, listen for when isBusy changes to false.
                        _this.reactionDisposer = reaction(someBusy, function () {
                            _this.disposeReaction();
                            isValid = allValid_1();
                            if (codeModal_1 !== null && isValid) {
                                codeModal_1.close();
                            }
                            resolve(isValid);
                        });
                        return;
                    }
                }
            }
            resolve(isValid);
        });
    };
    Form.prototype.disposeReaction = function () {
        if (this.reactionDisposer) {
            this.reactionDisposer();
            this.reactionDisposer = undefined;
        }
    };
    Form.prototype.componentWillUnmount = function () {
        this.disposeReaction();
    };
    Form.prototype.render = function () {
        var _this = this;
        var _a = this.props, validationDisplayMode = _a.validationDisplayMode, allReadOnly = _a.allReadOnly, allDisabled = _a.allDisabled, model = _a.model, showSummaryModal = _a.showSummaryModal, invalidSummaryTitle = _a.invalidSummaryTitle, invalidSummaryText = _a.invalidSummaryText, native = __rest(_a, ["validationDisplayMode", "allReadOnly", "allDisabled", "model", "showSummaryModal", "invalidSummaryTitle", "invalidSummaryText"]);
        return (React.createElement("form", __assign({}, native, { onSubmit: function (e) { return _this.onSubmit(e); } }),
            React.createElement(FormContextProvider, { validationDisplayMode: validationDisplayMode !== null && validationDisplayMode !== void 0 ? validationDisplayMode : Misc.Settings.form.validationDisplayMode, allReadOnly: allReadOnly, allDisabled: allDisabled }, (this.props.children instanceof Function) ? this.props.children(model, model.meta) : this.props.children)));
    };
    Form = __decorate([
        observer
    ], Form);
    return Form;
}(React.Component));
export default Form;
