import { __assign, __awaiter, __decorate, __extends, __generator, __metadata, __rest } from "tslib";
import * as React from 'react';
import { GridContext } from './Grid';
import NeoButton from '../Button';
import { Misc, Utils } from '@singularsystems/neo-core';
import ComponentBase from '../ComponentBase';
import { ColumnHelper } from './ColumnHelper';
import { observer } from "mobx-react";
var ButtonColumn = /** @class */ (function (_super) {
    __extends(ButtonColumn, _super);
    function ButtonColumn(props) {
        var _this = _super.call(this, props) || this;
        _this.onDelete = _this.onDelete.bind(_this);
        return _this;
    }
    ButtonColumn.prototype.onFirstRender = function () {
        this.section = this.context.section;
        this.item = this.context.currentItem;
    };
    Object.defineProperty(ButtonColumn.prototype, "itemAsModel", {
        get: function () {
            var _a;
            var item = (_a = this.props.item) !== null && _a !== void 0 ? _a : this.item;
            if ((item === null || item === void 0 ? void 0 : item.entityIdentifier) !== undefined) {
                return item;
            }
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    ButtonColumn.prototype.onDelete = function () {
        return __awaiter(this, void 0, void 0, function () {
            var item, list, mustRemove;
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        item = this.itemAsModel;
                        list = this.context.dataView.rawItems;
                        if (!(item.isNew === false && ((_a = this.props.deleteButton) === null || _a === void 0 ? void 0 : _a.onDeleteExisting) !== undefined)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.props.deleteButton.onDeleteExisting(item)];
                    case 1:
                        mustRemove = _b.sent();
                        if (mustRemove) {
                            list.removeWithoutTracking(item);
                        }
                        return [3 /*break*/, 3];
                    case 2:
                        if (list.toggleRemove !== undefined) {
                            list.toggleRemove(item);
                        }
                        else {
                            this.context.dataView.rawItems.remove(item);
                        }
                        _b.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    ButtonColumn.prototype.render = function () {
        var _a, _b;
        this.onRender();
        var _c = this.props, item = _c.item, label = _c.label, showDefaultButtonText = _c.showDefaultButtonText, buttonSize = _c.buttonSize, showDelete = _c.showDelete, editButton = _c.editButton, deleteButton = _c.deleteButton, viewButton = _c.viewButton, childAlignment = _c.childAlignment, hideBelow = _c.hideBelow, hideAbove = _c.hideAbove, alignment = _c.alignment, domProps = __rest(_c, ["item", "label", "showDefaultButtonText", "buttonSize", "showDelete", "editButton", "deleteButton", "viewButton", "childAlignment", "hideBelow", "hideAbove", "alignment"]);
        domProps.className = Utils.joinWithSpaces("neo-grid-button-column", domProps.className);
        domProps.className = ColumnHelper.addCommonClasses(domProps.className, this.props);
        var buttonColumnDefaults = Misc.Settings.grid.buttonColumnButtons;
        var editButtonDefaults = __assign(__assign({}, buttonColumnDefaults), Misc.Settings.grid.editButton);
        var deleteButtonDefaults = __assign(__assign({}, buttonColumnDefaults), Misc.Settings.grid.deleteButton);
        var viewButtonDefaults = __assign(__assign({}, buttonColumnDefaults), Misc.Settings.grid.viewButton);
        if (!showDefaultButtonText) {
            editButtonDefaults.text = "";
            deleteButtonDefaults.text = "";
            viewButtonDefaults.text = "";
        }
        if (buttonSize) {
            editButtonDefaults.size = buttonSize;
            deleteButtonDefaults.size = buttonSize;
            viewButtonDefaults.size = buttonSize;
        }
        var editButtonOptions = editButton ? Utils.populatePartialWithDefaults(editButton, editButtonDefaults) : null;
        var deleteButtonOptions = (deleteButton || showDelete) ? Utils.populatePartialWithDefaults(deleteButton, deleteButtonDefaults) : null;
        var viewButtonOptions = viewButton ? Utils.populatePartialWithDefaults(viewButton, viewButtonDefaults) : null;
        if (deleteButtonOptions) {
            if (!deleteButtonOptions.onClick) {
                deleteButtonOptions.onClick = this.onDelete;
            }
            if ((_a = this.itemAsModel) === null || _a === void 0 ? void 0 : _a.isSelfDeleting) {
                deleteButtonOptions.icon = Misc.Settings.icons.unDelete;
                (_b = deleteButtonOptions.tooltip) !== null && _b !== void 0 ? _b : (deleteButtonOptions.tooltip = "Undo delete");
            }
        }
        var buttons = React.createElement(React.Fragment, null,
            viewButtonOptions && React.createElement(NeoButton, __assign({}, viewButtonOptions)),
            editButtonOptions && React.createElement(NeoButton, __assign({}, editButtonOptions)),
            deleteButtonOptions && React.createElement(NeoButton, __assign({}, deleteButtonOptions)));
        var children = this.props.children instanceof Function ? this.props.children() : this.props.children;
        return (this.section !== "body" ?
            React.createElement("th", __assign({}, domProps), this.section === "header" && label) :
            React.createElement("td", __assign({}, domProps),
                childAlignment === "right" ? buttons : undefined,
                React.Children.map(children, function (c) { return setButtonProps(c); }),
                childAlignment !== "right" ? buttons : undefined));
    };
    ButtonColumn.isColumnComponent = true;
    ButtonColumn.contextType = GridContext;
    ButtonColumn = __decorate([
        observer,
        __metadata("design:paramtypes", [Object])
    ], ButtonColumn);
    return ButtonColumn;
}(ComponentBase));
export { ButtonColumn };
function setButtonProps(button) {
    if (button && button.type === NeoButton) {
        return React.cloneElement(button, __assign(__assign({}, Misc.Settings.grid.buttonColumnButtons), button.props));
    }
    return button;
}
