/**
 * Watches for when a table header scrolls out of view.
 * When it does, adds a cloned header that is fixed at the top of the scrolling element.
 */
export declare class StickyTableHeader {
    private table;
    private fixedElement?;
    /**
     * Creates the instance, and begins listening for scroll events.
     * @param table The table to add sticky headers to.
     * @param fixedElement A fixed element at the top of the screen to use as an anchor, rather than the top of the screen itself.
     */
    constructor(table: HTMLTableElement, fixedElement?: HTMLElement | undefined);
    private domObserver;
    private originalWidths;
    /** Parent element that may scroll, requiring a position update. */
    private scrollElement?;
    /** Fixed height parent element that the grid scrolls within. */
    private relativeParent?;
    private header;
    private headerCopy?;
    private headerHeight;
    private handleScroll;
    private removeCopy;
    private onPossibleContentChange;
    private syncColumnWidths;
    private storeColumnWidths;
    private restoreColumnWidths;
    dispose(): void;
    /**
     * Returns the first parent element that has a scrollbar.
     */
    private static getParentScrollElement;
}
