import { __assign, __decorate, __extends, __metadata, __rest } from "tslib";
import { AppServices, Misc, Utils } from "@singularsystems/neo-core";
import React from 'react';
import { NeoReactTypes } from "../Modules/Types";
import { observer } from "mobx-react";
/**
 * Renders an anchor tag with an href of '/' unless otherwise specified.
 * The onClick event will call preventDefault() so that the anchor acts like a button instead of a link to a url.
 */
var Link = /** @class */ (function (_super) {
    __extends(Link, _super);
    function Link(props) {
        var _this = _super.call(this, props) || this;
        _this.onClick = _this.onClick.bind(_this);
        _this.basePath = Misc.Globals.appService.get(NeoReactTypes.Routing.GlobalRoutingState).basePath;
        return _this;
    }
    Link.prototype.onClick = function (e) {
        if (this.props.onClick) {
            this.props.onClick(e);
        }
        if (!e.defaultPrevented) {
            e.preventDefault();
            if (this.props.to !== undefined) {
                Misc.Globals.appService.get(AppServices.NeoTypes.Routing.NavigationHelper).navigateInternal(this.props.to);
            }
        }
    };
    Link.prototype.render = function () {
        var _a = this.props, to = _a.to, href = _a.href, isNav = _a.isNav, exact = _a.exact, rest = __rest(_a, ["to", "href", "isNav", "exact"]);
        href !== null && href !== void 0 ? href : (href = "/");
        if (to !== undefined) {
            if (to.startsWith("/") && !to.startsWith(this.basePath)) {
                to = this.basePath + to;
            }
            href = to;
            if (isNav) {
                // Cause re-render if current route changes.
                void Misc.Globals.appService.get(NeoReactTypes.Routing.GlobalRoutingState).currentRoute;
                var path = window.location.pathname.toLowerCase();
                var toLc = to.toLowerCase();
                if (path.endsWith("/")) {
                    path = path.substring(0, path.length - 1);
                }
                var isMatch = exact === true ?
                    toLc === path :
                    path.startsWith(toLc);
                if (isMatch) {
                    rest.className = Utils.joinWithSpaces(rest.className, "active");
                }
            }
        }
        return React.createElement("a", __assign({}, rest, { href: href, onClick: this.onClick }), this.props.children);
    };
    Link = __decorate([
        observer,
        __metadata("design:paramtypes", [Object])
    ], Link);
    return Link;
}(React.Component));
export default Link;
