import React from 'react';
import { Data, Model, ValueObject } from '@singularsystems/neo-core';
import { IPagerControlsProps } from './BasicPagerControls';
import { DataViewContextParams } from '../../React/DataView';
export interface IPagerBaseProps<TCriteria extends ValueObject, TModel extends Model.IBindableBase> extends Omit<React.HTMLProps<HTMLDivElement>, "children"> {
    /** Page manager which controls this component. */
    pageManager: Data.PageManager<TCriteria, TModel>;
    /** Child can either be a component which works with a dataview (like Neo.Grid), or a callback which will be passed the items of the current page. */
    children: React.ReactNode | ((items: TModel[]) => React.ReactNode);
    /** Where to put the page controls? */
    pageControls?: "none" | "top" | "bottom" | "topAndBottom";
    /** True if the built in loader component should be hidden. */
    suppressLoader?: boolean;
    /** Properties to be passed to the pager controls. */
    pageControlProps?: Partial<IPagerControlsProps>;
    /** Set to false if you don't want the child control to be hidden before data is fetched for the first time. */
    hideBeforeLoad?: boolean;
}
export default class Pager<TCriteria extends ValueObject, TModel extends Model.IBindableBase> extends React.Component<IPagerBaseProps<TCriteria, TModel>> {
    dataViewContext: DataViewContextParams<TModel>;
    constructor(props: IPagerBaseProps<TCriteria, TModel>);
    componentDidMount(): void;
    componentDidUpdate(): void;
    render(): React.JSX.Element | null;
}
