import React from 'react';
import { IBindableEditorProps } from './PropTypes';
import ComponentBase from './ComponentBase';
import { FormContext } from './FormContext';
export interface IRadioListSpecificProps<T> {
    /** List of items to show. */
    items?: T[];
    /** Enum to show as a radio button list. */
    enumType?: any;
    /** True if the radio buttons should be displayed inline (horizontally). */
    inline?: boolean;
    /** Property name to use for the id. Only applies when using items. */
    valueMember?: keyof T;
    /** Property name to use for the radio button text. Only applies when using items. */
    displayMember?: keyof T;
    onItemSelected?: (item: T) => void;
    /** An array of tooltips, one per item in the order of the items. */
    tooltips?: (string | undefined)[];
}
export interface IRadioListProps<T> extends IBindableEditorProps<HTMLSelectElement> {
    radioList: IRadioListSpecificProps<T>;
}
export default class RadioList<T> extends ComponentBase<IRadioListProps<T>> {
    static contextType: React.Context<import("./FormContext").FormContextParams>;
    context: React.ContextType<typeof FormContext>;
    private previousList?;
    private previousEnumType?;
    private items;
    private preRender;
    private createEnumList;
    private onItemSelected;
    render(): React.JSX.Element;
}
