import { __assign, __extends, __rest } from "tslib";
import React from "react";
import { Components, Misc, Utils } from "@singularsystems/neo-core";
import { TabManager } from './TabManager';
import { TransitionContainer } from "../_Exports";
import { Transition } from "../Transitions/Transition";
var TabContextParams = /** @class */ (function () {
    function TabContextParams() {
        /**
         * True if the tab manager was passed in to the tab container.
         */
        this.isUserTabManager = false;
        this.inHeader = true;
    }
    return TabContextParams;
}());
export { TabContextParams };
export var TabContext = React.createContext(new TabContextParams());
var TabContainer = /** @class */ (function (_super) {
    __extends(TabContainer, _super);
    function TabContainer(props) {
        var _this = _super.call(this, props) || this;
        _this.tabContext = new TabContextParams();
        if (_this.props.tabManager) {
            _this.tabManager = _this.props.tabManager;
            _this.tabContext.isUserTabManager = true;
        }
        else {
            _this.tabManager = new TabManager("", true, _this.props.selectedTab);
        }
        _this.tabContext.tabManager = _this.tabManager;
        return _this;
    }
    TabContainer.prototype.componentDidMount = function () {
        this.tabManager.initialise();
    };
    TabContainer.prototype.componentWillUnmount = function () {
        this.tabManager.onUnmount();
    };
    TabContainer.prototype.render = function () {
        var _a = this.props, tabManager = _a.tabManager, selectedTab = _a.selectedTab, variant = _a.variant, fillMode = _a.fillMode, align = _a.align, verticalSizes = _a.verticalSizes, keepMounted = _a.keepMounted, animate = _a.animate, domProps = __rest(_a, ["tabManager", "selectedTab", "variant", "fillMode", "align", "verticalSizes", "keepMounted", "animate"]);
        this.tabContext.keepMounted = keepMounted;
        this.tabContext.animate = (animate !== null && animate !== void 0 ? animate : Misc.Settings.tabs.animate) !== false;
        if (!verticalSizes)
            verticalSizes = {};
        if (!verticalSizes.xs)
            verticalSizes.xs = 12;
        var isCardMode = (variant === "tabs" || !variant);
        var _b = Components.BootstrapHelpers.getInverseBreakpointClasses(verticalSizes), cols = _b.cols, inverseCols = _b.inverseCols;
        domProps.className = Utils.joinWithSpaces(domProps.className, "row neo-tab-container", isCardMode ? "neo-tab-card-mode" : undefined);
        return (React.createElement(TabContext.Provider, { value: this.tabContext },
            React.createElement("div", __assign({}, domProps),
                React.createElement("div", { className: "tabs-header " + cols },
                    React.createElement(TabSection, { isHeader: true, containerProps: this.props }, this.props.children)),
                React.createElement("div", { className: inverseCols },
                    React.createElement(TabSection, { isHeader: false, containerProps: this.props }, this.props.children)))));
    };
    return TabContainer;
}(React.Component));
export default TabContainer;
var TabSection = /** @class */ (function (_super) {
    __extends(TabSection, _super);
    function TabSection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TabSection.prototype.render = function () {
        var _a = this.props, isHeader = _a.isHeader, containerProps = _a.containerProps;
        this.context.inHeader = isHeader;
        var headerClassName = "nav";
        if (this.props.isHeader) {
            var variant = containerProps.variant;
            if (variant === "flat") {
                variant = "tabs";
            }
            if (containerProps.verticalSizes) {
                var breakpoint = Components.BootstrapHelpers.getFirstBreakpoint(containerProps.verticalSizes);
                if (breakpoint)
                    headerClassName += " flex-" + breakpoint + "-column";
            }
            if (variant !== "none") {
                headerClassName += " nav-" + (variant || "tabs");
            }
            if (containerProps.fillMode) {
                headerClassName += " nav-" + containerProps.fillMode;
            }
            if (containerProps.align) {
                headerClassName += " justify-content-" + containerProps.align;
            }
        }
        var transition = typeof containerProps.animate === "object" ? containerProps.animate : undefined;
        if (!transition && containerProps.verticalSizes) {
            transition = Transition.upDown;
        }
        return (this.props.isHeader ?
            React.createElement("ul", { className: headerClassName }, this.props.children) :
            React.createElement("div", { className: "tab-content" }, this.context.animate ?
                React.createElement(TransitionContainer, { transition: transition }, this.props.children) :
                this.props.children));
    };
    TabSection.contextType = TabContext;
    return TabSection;
}(React.PureComponent));
