import { Transition } from "./Transition";
export declare class TransitionController {
    /** Global default transition. */
    static transition: Transition;
    /** Allows transitions to be disabled for the whole app. */
    static disableTransitions: boolean;
    /**
     * Creates a transition controller.
     * @param initialName Name of the panel to display initially. Not required if `isVisible` props are defined on the panels.
     * @param transition Transition type to use by default on all panels.
     * @param backTransition Transition type to use when going backwards.
     */
    constructor(initialName?: string, transition?: Transition, backTransition?: Transition);
    private _selectedName;
    defaultTransition: Transition;
    defaultBackTransition: Transition;
    private direction;
    private isFirstTransition;
    private isTransitioning;
    private panels;
    private panelStates;
    private containerElement?;
    /**
     * Returns the name of the selected panel.
     */
    get selectedPanel(): string;
    /**
     * Begins a transition to the specified panel.
     * @param waitFor A promise on which to wait before showing the to panel. Use if data needs to be loaded.
     */
    transition(toPanelName: string, waitFor?: Promise<any>): void;
    forward(toPanelName?: string, waitFor?: Promise<any>): void;
    back(toPanelName?: string, waitFor?: Promise<any>): void;
    private doTransition;
    private beginRemove;
    private transitionElement;
}
