import ViewBase from '../Views/ViewBase';
import { RoutingState } from './RoutingState';
import { INavigationHelper as INeoNavigationHelper } from '@singularsystems/neo-core/dist/Routing/INavigationHelper';
import { RouteParameterObject } from './IRouteParameter';
import type { IPageLeaveHandler } from "./PageLeaveHandler";
export type ViewConstructor<TParams extends RouteParameterObject<TParams>> = new (...args: any) => ViewBase<any, TParams>;
export type ParamValues<TParams> = {
    [P in keyof TParams]?: number | string | null;
};
export interface INavigationHelper extends INeoNavigationHelper {
    /**
     * Navigates to the view, if the view component is found in the routes definition.
     * If the view is bound to multiple routes, the first one will be used.
     * @param view View component which inherits from view base.
     * @param paramValues View params to include in the url to pass to the view.
     */
    navigateToView<TParams extends RouteParameterObject<TParams>>(view: ViewConstructor<TParams>, paramValues?: ParamValues<TParams>): void;
    /**
     * Gets the full path for a view including view parameter values.
     * @param view View component which inherits from view base.
     * @param paramValues View params to include in the url to pass to the view.
     */
    getPathForView<TParams extends RouteParameterObject<TParams>>(view: ViewConstructor<TParams>, paramValues?: ParamValues<TParams>): string;
    /**
     * Gets the path of the currently rendered view, without parameter components.
     */
    getCurrentViewPath(): string;
}
export default class NavigationHelper implements INavigationHelper {
    private routingState;
    private pageLeaveHandler;
    constructor(routingState: RoutingState, pageLeaveHandler: IPageLeaveHandler);
    private prevUrls;
    private isPrompting;
    navigateInternal(url: string, replace?: boolean): Promise<boolean>;
    navigateToView<TParams extends RouteParameterObject<TParams>>(view: ViewConstructor<TParams>, paramValues?: ParamValues<TParams>): void;
    getPathForView<TParams extends RouteParameterObject<TParams>>(view: ViewConstructor<TParams>, paramValues?: ParamValues<TParams>): string;
    getCurrentViewPath(): string;
    private historyPopState;
    private windowUnload;
    private beforeLocationChanged;
}
