import { __awaiter, __decorate, __generator, __metadata, __param } from "tslib";
import { ViewParameterList } from './ViewParameterList';
import { injectable, inject } from 'inversify';
import { NeoReactTypes } from '../Modules/Types';
import { RoutingState } from './RoutingState';
var NavigationHelper = /** @class */ (function () {
    function NavigationHelper(routingState, pageLeaveHandler) {
        this.routingState = routingState;
        this.pageLeaveHandler = pageLeaveHandler;
        // PrevUrls is required since the popstate event tells you the new path, not the previous path
        this.prevUrls = [];
        this.isPrompting = false;
        window.addEventListener("beforeunload", this.windowUnload.bind(this));
        window.addEventListener("popstate", this.historyPopState.bind(this));
    }
    NavigationHelper.prototype.navigateInternal = function (url_1) {
        return __awaiter(this, arguments, void 0, function (url, replace) {
            var basePath, prevUrl, path, route, leavePrompt, canNavigate;
            if (replace === void 0) { replace = false; }
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        basePath = this.routingState.basePath;
                        prevUrl = this.prevUrls.length === 0 ? undefined : this.prevUrls[this.prevUrls.length - 1];
                        // Special case for when the root url has parameters. E.g. instead of /home/dashboard it will be //dashboard.
                        while (url.startsWith("//")) {
                            url = url.substring(1);
                        }
                        path = url;
                        if (basePath) {
                            if (path.startsWith(basePath)) {
                                path = path.substring(basePath.length);
                            }
                            else {
                                url = basePath + url;
                            }
                        }
                        if (!(prevUrl !== url)) return [3 /*break*/, 5];
                        route = this.routingState.appRouteProvider.getRouteForPath(path);
                        if (!this.routingState.currentRoute) return [3 /*break*/, 4];
                        leavePrompt = this.beforeLocationChanged(route, false);
                        if (!(leavePrompt !== undefined)) return [3 /*break*/, 4];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, , 3, 4]);
                        this.isPrompting = true;
                        return [4 /*yield*/, this.pageLeaveHandler.onLeavePage(this.routingState.currentView, leavePrompt)];
                    case 2:
                        canNavigate = _a.sent();
                        if (!canNavigate) {
                            return [2 /*return*/, false];
                        }
                        return [3 /*break*/, 4];
                    case 3:
                        this.isPrompting = false;
                        return [7 /*endfinally*/];
                    case 4:
                        if (replace) {
                            if (this.prevUrls.length > 0) {
                                this.prevUrls[this.prevUrls.length - 1] = url;
                            }
                            window.history.replaceState(null, "", url);
                        }
                        else {
                            this.prevUrls.push(url);
                            window.history.pushState(null, "", url);
                        }
                        this.routingState.updateRoute(route);
                        _a.label = 5;
                    case 5: return [2 /*return*/, true];
                }
            });
        });
    };
    NavigationHelper.prototype.navigateToView = function (view, paramValues) {
        this.navigateInternal(this.getPathForView(view, paramValues));
    };
    NavigationHelper.prototype.getPathForView = function (view, paramValues) {
        var basePath = this.routingState.appRouteProvider.getPathForComponent(view);
        if (basePath !== undefined) {
            if (paramValues) {
                return ViewParameterList.getPathForValues(basePath, view.params, paramValues);
            }
            else {
                return basePath;
            }
        }
        else {
            throw Error("Cannot find route for component " + view.name);
        }
    };
    NavigationHelper.prototype.getCurrentViewPath = function () {
        if (this.routingState.currentRoute) {
            return this.routingState.currentRoute.route.path;
        }
        else {
            return "";
        }
    };
    NavigationHelper.prototype.historyPopState = function (e) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.isPrompting;
                        if (_a) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.navigateInternal(window.location.pathname, true)];
                    case 1:
                        _a = !(_b.sent());
                        _b.label = 2;
                    case 2:
                        if (_a) {
                            // cancel navigation by pushing the previous state back into history.
                            window.history.pushState(null, "", this.prevUrls[this.prevUrls.length - 1]);
                        }
                        else {
                            // remove our state to match browsers state.
                            this.prevUrls.pop();
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    NavigationHelper.prototype.windowUnload = function (e) {
        var leavePrompt = this.beforeLocationChanged(null, true);
        if (leavePrompt !== undefined) {
            this.pageLeaveHandler.onWindowUnload(this.routingState.currentView, leavePrompt, e);
        }
    };
    NavigationHelper.prototype.beforeLocationChanged = function (newRoute, isUnload) {
        var _a;
        var currentRoute = this.routingState.currentRoute;
        var changeProps = {
            isNewView: !currentRoute || currentRoute.route !== (newRoute === null || newRoute === void 0 ? void 0 : newRoute.route),
            isUnload: isUnload !== null && isUnload !== void 0 ? isUnload : false,
            currentView: (_a = this.routingState.currentView) !== null && _a !== void 0 ? _a : undefined,
            newRoute: newRoute === null || newRoute === void 0 ? void 0 : newRoute.route,
            params: newRoute === null || newRoute === void 0 ? void 0 : newRoute.params
        };
        if (this.routingState.currentView) {
            this.routingState.currentView.beforeRouteChanged(changeProps);
            if (changeProps.leavePrompt === undefined && (changeProps.isNewView || changeProps.isUnload)) {
                changeProps.leavePrompt = this.routingState.currentView.onLeave();
            }
        }
        if (changeProps.isNewView) {
            this.pageLeaveHandler.beforeLeavePage(changeProps);
        }
        return changeProps.leavePrompt;
    };
    NavigationHelper = __decorate([
        injectable(),
        __param(0, inject(NeoReactTypes.Routing.GlobalRoutingState.symbol)),
        __param(1, inject(NeoReactTypes.Routing.PageLeaveHandler.symbol)),
        __metadata("design:paramtypes", [RoutingState, Object])
    ], NavigationHelper);
    return NavigationHelper;
}());
export default NavigationHelper;
