import { __extends } from "tslib";
import * as React from 'react';
import { AppServices, Misc } from '@singularsystems/neo-core';
import { NeoReactTypes } from '../Modules/Types';
import { reaction } from "mobx";
var RouteView = /** @class */ (function (_super) {
    __extends(RouteView, _super);
    function RouteView(props, routeProvider) {
        var _this = _super.call(this, props) || this;
        _this.routeProvider = routeProvider;
        _this.hasMounted = false;
        _this.securityService = Misc.Globals.appService.get(AppServices.NeoTypes.Security.SecurityService);
        _this.routeSecurityService = Misc.Globals.appService.get(AppServices.NeoTypes.Routing.RouteSecurityService);
        _this.routingState = Misc.Globals.appService.get(NeoReactTypes.Routing.GlobalRoutingState);
        _this.state = {};
        _this.routingState.appRouteProvider = routeProvider;
        reaction(function () { return _this.routingState.currentRouteBase; }, function () { return _this.forceUpdate(); });
        return _this;
    }
    RouteView.prototype.componentDidMount = function () {
        this.hasMounted = true;
        var basePath = this.routingState.basePath;
        var path = window.location.pathname + window.location.search;
        if (basePath && !window.location.pathname.startsWith(basePath)) {
            console.warn("Base path is ".concat(basePath, " but the app is being served at ").concat(window.location.pathname, "."));
        }
        Misc.Globals.appService.get(NeoReactTypes.Routing.NavigationHelper).navigateInternal(path, true);
    };
    RouteView.prototype.render = function () {
        return this.routingState.currentRoute ?
            this.getRouteComponent(this.routingState.currentRoute) :
            (this.hasMounted && this.routeProvider.notFoundComponent ? React.createElement(this.routeProvider.notFoundComponent) : null);
    };
    RouteView.prototype.getRouteComponent = function (match) {
        var route = match.route;
        var path = match.path + window.location.search;
        if (!route.allowAnonymous) {
            if (this.securityService.isAuthenticated) {
                if (!this.isAuthorised(route)) {
                    //Render forbidden
                    return this.getForbiddenComponent(route);
                }
            }
            else {
                //Redirect
                void this.securityService.authentication.beginSignIn(path);
                return this.getSigningInComponent(route);
            }
        }
        return this.getAuthorisedComponent(route, path);
    };
    /**
     * Creates the routes component after authentication and authorisation checks have been done.
     * This can be used to override all routes to display pages like terms and conditions.
     */
    RouteView.prototype.getAuthorisedComponent = function (route, path) {
        return React.createElement(route.component, null);
    };
    /**
     * Checks if the user is authorised to access this route.
     * Component from getForbiddenComponent() is returned if not.
     */
    RouteView.prototype.isAuthorised = function (route) {
        return this.routeSecurityService.routeAllowed(route);
    };
    return RouteView;
}(React.Component));
export default RouteView;
