import { __decorate, __metadata, __param } from "tslib";
import { AppServices } from '@singularsystems/neo-core';
import { inject, injectable } from 'inversify';
import { computed, makeObservable, observable, runInAction } from 'mobx';
var RoutingState = /** @class */ (function () {
    function RoutingState(config) {
        /** Gets the route which last matched */
        this.currentRoute = null;
        /** Base route which last matched. This will not change when view parameters are updated. */
        this.currentRouteBase = null;
        this.direction = 0;
        /** Returns the current view instance. */
        this.currentView = null;
        makeObservable(this);
        this.basePath = config.baseUrl;
    }
    Object.defineProperty(RoutingState.prototype, "currentViewFullScreen", {
        get: function () {
            var _a;
            return ((_a = this.currentView) === null || _a === void 0 ? void 0 : _a.fullScreen) === true;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(RoutingState.prototype, "taskRunner", {
        get: function () {
            var _a;
            return (_a = this.currentView) === null || _a === void 0 ? void 0 : _a.taskRunner;
        },
        enumerable: false,
        configurable: true
    });
    /** @internal */
    RoutingState.prototype.updateRoute = function (match) {
        var _this = this;
        var _a, _b;
        var prevRoute = (_a = this.currentRoute) === null || _a === void 0 ? void 0 : _a.route;
        runInAction(function () {
            var _a;
            _this.currentRoute = match;
            _this.currentRouteBase = (_a = match === null || match === void 0 ? void 0 : match.route) !== null && _a !== void 0 ? _a : null;
        });
        if (prevRoute !== (match === null || match === void 0 ? void 0 : match.route)) {
            var prevIndex = prevRoute ? this.appRouteProvider.allRoutes.indexOf(prevRoute) : -1;
            var newIndex = match ? this.appRouteProvider.allRoutes.indexOf(match.route) : -1;
            this.direction = newIndex - prevIndex;
            if (((_b = this.currentView) === null || _b === void 0 ? void 0 : _b.constructor) !== (match === null || match === void 0 ? void 0 : match.route.component)) {
                this.currentView = null;
            }
            // Scroll back to top as if the browser had actually loaded a new page.
            window.scrollTo(0, 0);
        }
        else {
            // Same route, but parameters must have changed.
            if (this.currentView && this.currentRoute) {
                this.currentView.updateViewParams(this.currentRoute);
            }
        }
    };
    __decorate([
        observable.ref,
        __metadata("design:type", Object)
    ], RoutingState.prototype, "currentRoute", void 0);
    __decorate([
        observable.ref,
        __metadata("design:type", Object)
    ], RoutingState.prototype, "currentRouteBase", void 0);
    __decorate([
        observable.ref,
        __metadata("design:type", Object)
    ], RoutingState.prototype, "currentView", void 0);
    __decorate([
        computed,
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], RoutingState.prototype, "currentViewFullScreen", null);
    __decorate([
        computed,
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], RoutingState.prototype, "taskRunner", null);
    RoutingState = __decorate([
        injectable(),
        __param(0, inject(AppServices.NeoTypes.Config.ConfigModel.symbol)),
        __metadata("design:paramtypes", [Object])
    ], RoutingState);
    return RoutingState;
}());
export { RoutingState };
