import { ViewParameter, IViewParameter } from './ViewParameter';
import { RouteParameterObject } from './IRouteParameter';
import { BreadCrumbItem } from './BreadCrumbItem';
import { IViewBase } from '../Views/ViewBase';
import { ParamValues } from "./NavigationHelper";
export type TransformParamsType<T> = {
    [P in keyof T]: IViewParameter;
};
interface ISetValuesOptions {
    /** True if the url should be set even if none of the values have changed. */
    force?: boolean;
    /** True if the url should be replaced instead of added to the history. This prevents the back button returning the params to the current state. */
    replace?: boolean;
}
export interface IViewParameterList {
    /**
     * Returns the breadcrumb items for all url parameters which have a value.
     */
    getBreadCrumbList(): BreadCrumbItem[];
}
export interface IViewParameterListOfT<TParams extends RouteParameterObject<TParams>> extends IViewParameterList {
    /**
     * Set the view params for each of the provided values. Only 1 navigation event will occur.
     * @param values New values.
     */
    setValues(values: {
        [P in keyof TParams]?: number | string | string[] | null;
    }, options?: ISetValuesOptions | boolean): void;
}
export interface IViewParameterListInternal extends IViewParameterList {
    setUrl(viewParameter: ViewParameter | null, replace?: boolean): void;
    view?: IViewBase;
}
export declare class ViewParameterList<TParams extends RouteParameterObject<TParams>> implements IViewParameterListInternal, IViewParameterListOfT<TParams> {
    readonly view?: IViewBase | undefined;
    readonly params: ViewParameter[];
    private suppressNavigation;
    private hasSuppressedNavigation;
    constructor(routeParameterObject: TParams, view?: IViewBase | undefined);
    getBreadCrumbList(upTo?: ViewParameter): BreadCrumbItem[];
    private getQueryString;
    setValues(values: {
        [P in keyof TParams]?: number | string | string[] | null;
    }, options?: ISetValuesOptions | boolean): void;
    setUrl(viewParameter: ViewParameter | null, replace?: boolean): void;
    /**
     * Gets a url for an object with view params specified.
     * @param basePath Base path without parameters
     * @param values Object of param names and values.
     */
    static getPathForValues<TParams extends RouteParameterObject<TParams>>(basePath: string, params: TParams, values: ParamValues<TParams>): string;
}
export {};
